﻿using Microsoft.IdentityModel.Clients.ActiveDirectory;
using Microsoft.PowerBI.Api.V2;
using Microsoft.PowerBI.Api.V2.Models;
using System;
using System.Linq;
using System.Web.UI;
using PBIWebApp.Properties;
using Microsoft.Rest;
using System.Threading.Tasks;

namespace PBIWebApp
{
    /* NOTE: This code is for sample purposes only. In a production application, you could use a MVC design pattern.
    * In addition, you should provide appropriate exception handling and refactor authentication settings into 
    * a secure configuration. Authentication settings are hard-coded in the sample to make it easier to follow the flow of authentication. 
    * In addition, the sample uses a single web page so that all code is in one location. However, you could refactor the code into
    * your own production model.
    */
    public partial class _Default : Page
    {
        //string baseUri = Settings.Default.PowerBiDataset;

        private static readonly string AuthorityUrl = Settings.Default.AADAuthorityUri;
        private static readonly string ResourceUrl = Settings.Default.PowerBiAPIResource;
        private static readonly string ApplicationId = Settings.Default.ApplicationID;
        private static readonly string ApiUrl = Settings.Default.PowerBiApiUrl;
        private static readonly string WorkspaceId = Settings.Default.WorkspaceId;
        private static readonly string ReportId = Settings.Default.ReportId;
        private static readonly string Username = Settings.Default.pbiUsername;
        private static readonly string Password = Settings.Default.pbiPassword;

        protected void Page_Load(object sender, EventArgs e)
        {

        }

        protected void getReportButton_Click(object sender, EventArgs e)
        {
            try {
                GetReport().Wait();
              }
            catch (Exception ex)
            {
                errorLabel.Text = ex.ToString();
            }

        }


        // Gets a report based on the setting's ReportId and WorkspaceId.
        // If reportId or WorkspaceId are empty, it will get the first user's report.
        protected async Task<AuthenticationResult> GetReport()
        {
            // Create a user password cradentials.
            var credential = new UserPasswordCredential(Username, Password);
            // Authenticate using created credentials
            var authenticationContext = new AuthenticationContext(AuthorityUrl);
            var authenticationResult = await authenticationContext.AcquireTokenAsync(ResourceUrl, 
                ApplicationId, credential).ConfigureAwait(false);

            if (authenticationResult == null)
            {
                errorLabel.Text = "Authentication Failed.";  
            }

            using (var client = new PowerBIClient(new Uri(ApiUrl), new TokenCredentials(authenticationResult.AccessToken, "Bearer")))
            {
                Report report;

                // Settings' workspace ID is not empty
                if (!string.IsNullOrEmpty(WorkspaceId))
                {
                    // Gets a report from the workspace.
                    report = GetReportFromWorkspace(client, WorkspaceId, ReportId);
                }
                // Settings' report and workspace Ids are empty, retrieves the user's first report.
                else if (string.IsNullOrEmpty(ReportId))
                {
                    report = client.Reports.GetReports().Value.FirstOrDefault();
                    AppendErrorIfReportNull(report, "No reports found. Please specify the target report ID and workspace in the applications settings.");
                }
                // Settings contains report ID. (no workspace ID)
                else
                {
                    report = client.Reports.GetReports().Value.FirstOrDefault(r => r.Id == ReportId);
                    AppendErrorIfReportNull(report, string.Format("Report with ID: '{0}' not found. Please check the report ID. For reports within a workspace with a workspace ID, add the workspace ID to the application's settings", ReportId));
                }

                // Generate Embed Token for reports without effective identities.
                GenerateTokenRequest generateTokenRequestParameters = new GenerateTokenRequest(accessLevel: "view");

                var tokenResponse = await client.Reports.GenerateTokenInGroupAsync(WorkspaceId, report.Id, generateTokenRequestParameters).ConfigureAwait(false);

                if (tokenResponse == null)
                {
                    errorLabel.Text = "Failed to generate embed token.";
                }

                if (report != null)
                {
                    accessToken.Value = tokenResponse.Token;
                    txtEmbedUrl.Text = report.EmbedUrl;
                    txtReportId.Text = report.Id;
                    txtReportName.Text = report.Name;
                }
                return authenticationResult;
            }
        }
        
        private Report GetReportFromWorkspace(PowerBIClient client, string WorkspaceId, string reportId)
        {
            // Gets the workspace by WorkspaceId.
            var workspaces = client.Groups.GetGroups();
            var sourceWorkspace = workspaces.Value.FirstOrDefault(g => g.Id == WorkspaceId);

            // No workspace with the workspace ID was found.
            if (sourceWorkspace == null)
            {
                errorLabel.Text = string.Format("Workspace with id: '{0}' not found. Please validate the provided workspace ID.", WorkspaceId);
                return null;
            }

            Report report = null;
            if (string.IsNullOrEmpty(reportId))
            {
                // Get the first report in the workspace.
                report = client.Reports.GetReportsInGroup(sourceWorkspace.Id).Value.FirstOrDefault();
                AppendErrorIfReportNull(report, "Workspace doesn't contain any reports.");
            }

            else
            {
                try
                {
                    // retrieve a report by the workspace ID and report ID.
                    report = client.Reports.GetReportInGroup(WorkspaceId, reportId);
                }

                catch(HttpOperationException)
                {
                    errorLabel.Text = string.Format("Report with ID: '{0}' not found in the workspace with ID: '{1}', Please check the report ID.", reportId, WorkspaceId);

                }
            }

            return report;
        }

        private void AppendErrorIfReportNull(Report report, string errorMessage)
        {
            if (report == null)
            {
                errorLabel.Text = errorMessage;
            }
        }
    }
}